﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace Magnetometer_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		Magnetometer openCh = null;
		formCleaner cleaner;
		Magnetometer mag = null;
		private TemperatureSensor temperatureSensor;
		private ErrorEventBox errorBox;
		private DataGraphBox xGraphBox, yGraphBox, zGraphBox;

		double ambientMagneticField = 1.0; //Unit Circle

		FastEventHandler<MagnetometerMagneticFieldChangeEventArgs> magEventHandler;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(Magnetometer ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		private void Form1_Load(object sender, EventArgs e) {

			errorBox = new ErrorEventBox(this);
			xGraphBox = new DataGraphBox(this, "Magnetic Field Strength");
			yGraphBox = new DataGraphBox(this, "Magnetic Field Strength");
			zGraphBox = new DataGraphBox(this, "Magnetic Field Strength");
			mag = openCh ?? openArgs.makeChannel<Magnetometer>();

			initMagFieldGraph();

			mag.Attach += magnetometer_attach;
			mag.Detach += magnetometer_detach;
			mag.Error += magnetometer_error;
			mag.PropertyChange += magnetometer_propertychange;

			// This causes the handler to get called without Invoke
			magEventHandler = new FastEventHandler<MagnetometerMagneticFieldChangeEventArgs>();
			mag.MagneticFieldChange += magEventHandler.Handler;
			magEventHandler.Event += fieldStrength_change;

			temperatureSensor = new TemperatureSensor();
			temperatureSensor.TemperatureChange += TemperatureSensor_TemperatureChange;
			temperatureSensor.Detach += TemperatureSensor_Detach;

			try {
				mag.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }

			try {
				string calibrationExamplePath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
				calibrationExamplePath += "\\CompassCalibrator.exe";

				if(System.IO.File.Exists(calibrationExamplePath))
				{
					calibrateButton.Visible = true;
				}
			}
			catch {}
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			mag.Attach -= magnetometer_attach;
			mag.Detach -= magnetometer_detach;
			mag.Error -= magnetometer_error;
			mag.PropertyChange -= magnetometer_propertychange;
			mag.MagneticFieldChange -= magEventHandler.Handler;
			mag.Close();

			temperatureSensor.TemperatureChange -= TemperatureSensor_TemperatureChange;
			temperatureSensor.Detach -= TemperatureSensor_Detach;
			temperatureSensor.Close();
		}

		private void TemperatureSensor_Detach(object sender, DetachEventArgs e)
		{
			temperatureReading.Text = "Detached";
		}

		private void TemperatureSensor_TemperatureChange(object sender, TemperatureSensorTemperatureChangeEventArgs e)
		{
			double targetTemp = 50;
			temperatureReading.Text = e.Temperature.ToString() + "°C";
			switch (((TemperatureSensor)sender).DeviceID)
			{
				case DeviceID.PN_MOT0109:
					targetTemp = 45;
					break;
			}
			if (e.Temperature > (targetTemp - 0.2) && e.Temperature < (targetTemp + 0.2))
				temperatureReading.ForeColor = Color.Green;
			else
				temperatureReading.ForeColor = Color.Red;
		}

		void magnetometer_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			settingsBox.Visible = true;
			outputBox.Visible = true;
			magFieldGraphBox.Visible = true;

			Magnetometer attachedDevice = (Magnetometer)sender;

			xGraphBox.SetTitle(DataGraphBox.GenerateTitle("X Axis", (Phidget)attachedDevice));
			yGraphBox.SetTitle(DataGraphBox.GenerateTitle("Y Axis", (Phidget)attachedDevice));
			zGraphBox.SetTitle(DataGraphBox.GenerateTitle("Z Axis", (Phidget)attachedDevice));

			try
			{
				heatingEnabledBox.Checked = attachedDevice.HeatingEnabled;
				heatingEnabledPanel.Visible = true;

				//Use the temperature sensor specifically from the attached device
				try
				{ //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
					temperatureSensor.Channel = attachedDevice.Channel; //selects the channel on the device to open
					temperatureSensor.DeviceSerialNumber = attachedDevice.DeviceSerialNumber; //selects the device or hub to open
					temperatureSensor.HubPort = attachedDevice.HubPort; //selects the port on the hub to open
					temperatureSensor.IsHubPortDevice = attachedDevice.IsHubPortDevice; //is the device a port on a VINT hub?
					if (attachedDevice.IsRemote) //are we trying to open a remote device?
						temperatureSensor.IsRemote = true;
					else
						temperatureSensor.IsLocal = true;

					temperatureSensor.Open(); //open the device specified by the above parameters
				}
				catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
			} catch {
				heatingEnabledPanel.Visible = false;
			}

			try {
				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString();
				dataIntervalTrk.Minimum = attachedDevice.MinDataInterval;
				dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString();
				dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval;
				dataIntervalTrk.trkText = attachedDevice.DataInterval.ToString();
				dataIntervalTrk.Value = attachedDevice.DataInterval;

				changeTriggerTrk.Unit = "";
				changeTriggerTrk.minTxt = attachedDevice.MinMagneticFieldChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinMagneticFieldChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxMagneticFieldChangeTrigger;
				changeTriggerTrk.maxTxt = attachedDevice.MaxMagneticFieldChangeTrigger.ToString();
				changeTriggerTrk.trkText = attachedDevice.MagneticFieldChangeTrigger.ToString();
				changeTriggerTrk.Value = attachedDevice.MagneticFieldChangeTrigger;

			} catch (PhidgetException ex) {
				errorBox.addMessage("Error initializing device: " + ex.Message);
			}

			updateGraphDataInterval();

			guiUpdateTimer.Start();
		}

		void magnetometer_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			guiUpdateTimer.Stop();
			_mag[0] = 0;
			_mag[1] = 0;
			_mag[2] = 0;
			_timestamp = 0;

			temperatureSensor.Close();

			phidgetInfoBox1.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			outputBox.Visible = false;
			magFieldGraphBox.Visible = false;
			calibrationStatusTxt.Visible = false;
		}

		void magnetometer_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation) {
				xGraphBox.addGap();
				yGraphBox.addGap();
				zGraphBox.addGap();
			}
		}

		private void magnetometer_propertychange(object sender, PropertyChangeEventArgs e)
		{

			switch (e.PropertyName)
			{
			case "DataInterval":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
				{
					dataIntervalTrk.Value = mag.DataInterval;
					updateGraphDataInterval();
				}
				break;
			case "DataRate":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
				{
					dataIntervalTrk.Value = mag.DataRate;
					updateGraphDataInterval();
				}
				break;
			case "MagneticFieldChangeTrigger":
				changeTriggerTrk.Value = mag.MagneticFieldChangeTrigger;
				break;
			case "HeatingEnabled":
				heatingEnabledBox.Checked = mag.HeatingEnabled;
				break;
			}
		}

		double[] _mag = { 0, 0, 0 };
		double _timestamp = 0;
		private void guiUpdateTimer_Tick(object sender, EventArgs e) {
			xaxistxt.Text = _mag[0].ToString();
			yaxistxt.Text = _mag[1].ToString();
			zaxistxt.Text = _mag[2].ToString();
			timestamptxt.Text = _timestamp.ToString() + " ms";

			try {
				drawMagFieldGraph();
			} catch { }
		}

		void fieldStrength_change(object sender, Phidget22.Events.MagnetometerMagneticFieldChangeEventArgs e) {

			_mag[0] = e.MagneticField[0];
			_mag[1] = e.MagneticField[1];
			_mag[2] = e.MagneticField[2];
			_timestamp = e.Timestamp;

			xGraphBox.addData(e.MagneticField[0], null, e.Timestamp);
			yGraphBox.addData(e.MagneticField[1], null, e.Timestamp);
			zGraphBox.addData(e.MagneticField[2], null, e.Timestamp);
		}

		private void updateGraphDataInterval() {
			try {
				if (mag.MagneticFieldChangeTrigger == 0) {
					xGraphBox.DataInterval = mag.DataInterval;
					yGraphBox.DataInterval = mag.DataInterval;
					zGraphBox.DataInterval = mag.DataInterval;
				} else {
					xGraphBox.DataInterval = null;
					yGraphBox.DataInterval = null;
					zGraphBox.DataInterval = null;
				}
			} catch (PhidgetException ex) { errorBox.addMessage("Error updating graph data interval: " + ex.Message); }
		}

		private void dataIntervalTrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					mag.DataInterval = (int)dataIntervalTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) {
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}

		private void dataIntervalTrk_LogMouseUp(object sender, MouseEventArgs e) {
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					mag.DataInterval = (int)Math.Round(dataIntervalTrk.Value);
					updateGraphDataInterval();
				}
				catch (Exception ex)
				{
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					mag.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex)
				{
					errorBox.addMessage("Error setting data rate: " + ex.Message);
				}
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}

		private void changeTriggertrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					mag.MagneticFieldChangeTrigger = changeTriggerTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) {
					errorBox.addMessage("Error setting change trigger: " + ex.Message);
				}
			}
		}

		private void changeTriggerTrk_LogMouseUp(object sender, MouseEventArgs e) {
			try {
				mag.MagneticFieldChangeTrigger = changeTriggerTrk.Value;
				updateGraphDataInterval();
			} catch (Exception ex) {
				errorBox.addMessage("Error setting change trigger: " + ex.Message);
			}
		}

		private void setCompassParamsButton_Click(object sender, EventArgs e) {
			ParameterInputBox tib = new ParameterInputBox();
			DialogResult result = tib.ShowDialog();

			if (result == DialogResult.OK) {

				if (tib.parameters.Length == 0) {
					try {
						mag.ResetCorrectionParameters();
						mag.SaveCorrectionParameters();
						calibrationStatusTxt.Text = "Un-Calibrated";
						calibrationStatusTxt.ForeColor = Color.Red;
						calibrationStatusTxt.Visible = true;
					} catch (PhidgetException ex) { errorBox.addMessage("Error resetting calibration: " + ex.Message); }
					return;
				}

				string[] paramVals = tib.parameters.Split(new char[] { ',' });
				if (paramVals.Length == 13) {
					try {
						mag.SetCorrectionParameters(
							double.Parse(paramVals[0]),
							double.Parse(paramVals[1]),
							double.Parse(paramVals[2]),
							double.Parse(paramVals[3]),
							double.Parse(paramVals[4]),
							double.Parse(paramVals[5]),
							double.Parse(paramVals[6]),
							double.Parse(paramVals[7]),
							double.Parse(paramVals[8]),
							double.Parse(paramVals[9]),
							double.Parse(paramVals[10]),
							double.Parse(paramVals[11]),
							double.Parse(paramVals[12])
						);
						mag.SaveCorrectionParameters();
						ambientMagneticField = double.Parse(paramVals[0]);
						calibrationStatusTxt.Text = "Calibrated";
						calibrationStatusTxt.ForeColor = Color.Green;
						calibrationStatusTxt.Visible = true;
					} catch (PhidgetException ex) { errorBox.addMessage("Error calibrating: " + ex.Message); }
				}
			}
		}


		private void xGraphButton_Click(object sender, EventArgs e) {
			if (!xGraphBox.Visible)
				xGraphBox.Show();
		}
		private void yGraphButton_Click(object sender, EventArgs e) {
			if (!yGraphBox.Visible)
				yGraphBox.Show();
		}

		private void zGraphButton_Click(object sender, EventArgs e) {
			if (!zGraphBox.Visible)
				zGraphBox.Show();
		}

		private void dataIntervalTrk_SetClick(object sender, EventArgs e)
		{
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					mag.DataInterval = (int)dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					mag.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
			}

		}

		private void dataIntervalTrk_SetEnter(object sender, KeyEventArgs e)
		{
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				if (e.KeyCode == Keys.Enter)
				{
					try
					{
						mag.DataInterval = (int)dataIntervalTrk.Value;
						updateGraphDataInterval();
					}
					catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				}
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				if (e.KeyCode == Keys.Enter)
				{
					try
					{
						mag.DataRate = dataIntervalTrk.Value;
						updateGraphDataInterval();
					}
					catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
				}
			}
		}

		private void changeTriggerTrk_SetClick(object sender, EventArgs e) {
			try {
				mag.MagneticFieldChangeTrigger = changeTriggerTrk.Value;
				updateGraphDataInterval();
			} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
		}

		private void changeTriggerTrk_SetEnter(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				try {
					mag.MagneticFieldChangeTrigger = changeTriggerTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}

		#region graph
		private BufferedGraphicsContext magFieldGraphicsContext;
		private BufferedGraphics magFieldGraphicsBuffer;
		private Bitmap magFieldDrawingSurface;

		private Rectangle magFieldCircleRectangle, magFieldBoundsRectangle;
		private Pen magFieldXYAxisPen, magFieldCirclePen;

		private void calibrateButton_Click(object sender, EventArgs e)
		{
			try
			{
				List<String> commandLineArgs = new List<string>();
				if (mag.DeviceClass == DeviceClass.VINT)
				{
					commandLineArgs.Add("-v");
					commandLineArgs.Add(mag.HubPort.ToString());
					commandLineArgs.Add("-n");
					commandLineArgs.Add(mag.DeviceSerialNumber.ToString());
				}
				else {
					commandLineArgs.Add("-n");
					commandLineArgs.Add(mag.DeviceSerialNumber.ToString());
				}
				if (mag.IsRemote)
				{
					commandLineArgs.Add("-s");
					commandLineArgs.Add(mag.ServerName.ToString());
				}

				String args = String.Join(" ", commandLineArgs.ToArray());

				System.Diagnostics.Process exampleApp = new System.Diagnostics.Process();
				string path = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
				exampleApp.StartInfo.FileName = path + "\\CompassCalibrator.exe";
				exampleApp.StartInfo.Arguments = args;
				exampleApp.Start();
				this.Close();
			}
			catch (Exception ex)
			{
				MessageBox.Show(this, "Error running compass calibrator: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
			}
		}

        private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e)
        {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.Maximum = mag.MaxDataRate;
					dataIntervalTrk.Minimum = mag.MinDataRate;
					dataIntervalTrk.maxTxt = mag.MaxDataRate.ToString();
					dataIntervalTrk.minTxt = mag.MinDataRate.ToString("F3");
					dataIntervalTrk.trkText = mag.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = mag.DataRate;
				}
				catch (Exception ex)
				{ errorBox.addMessage("Error setting data rate: " + ex.Message); }

			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					dataIntervalTrk.isIntValue = true;
					dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.Maximum = mag.MaxDataInterval;
					dataIntervalTrk.Minimum = mag.MinDataInterval;
					dataIntervalTrk.maxTxt = mag.MaxDataInterval.ToString();
					dataIntervalTrk.minTxt = mag.MinDataInterval.ToString();
					dataIntervalTrk.trkText = mag.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = mag.DataInterval;
				}
				catch (Exception ex)
				{ errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}

        private void heatingEnabledBox_CheckedChanged(object sender, EventArgs e)
		{
			mag.HeatingEnabled = heatingEnabledBox.Checked;
		}

		private float magFieldCircleDiameter, magFieldCircleRadius;

		private void initMagFieldGraph() {

			//initialize our graphical representation of the compasseration data
			magFieldDrawingSurface = new Bitmap(magFieldView.ClientRectangle.Width, magFieldView.ClientRectangle.Height, System.Drawing.Imaging.PixelFormat.Format24bppRgb);

			magFieldBoundsRectangle = new Rectangle(0, 0, magFieldDrawingSurface.Width, magFieldDrawingSurface.Height);
			magFieldBoundsRectangle.Inflate(-1, -1);

			float xCenter = (float)(magFieldBoundsRectangle.Width / 2.0);
			float yCenter = (float)(magFieldBoundsRectangle.Height / 2.0);

			magFieldXYAxisPen = new Pen(Color.Black, 2);
			magFieldCirclePen = new Pen(Color.DarkBlue, 2);

			magFieldCircleDiameter = (float)(magFieldDrawingSurface.Width / 1.1);
			magFieldCircleRadius = magFieldCircleDiameter / 2;

			magFieldCircleRectangle = new Rectangle((int)Math.Round(xCenter - magFieldCircleRadius), (int)Math.Round(yCenter - magFieldCircleRadius), (int)Math.Round(magFieldCircleDiameter), (int)Math.Round(magFieldCircleDiameter));
		}

		private void drawMagFieldGraph() {
			magFieldGraphicsContext = new BufferedGraphicsContext();
			magFieldGraphicsBuffer = magFieldGraphicsContext.Allocate(Graphics.FromImage(magFieldDrawingSurface), magFieldBoundsRectangle);
			magFieldGraphicsBuffer.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			magFieldGraphicsBuffer.Graphics.Clear(SystemColors.Control);

			float xCenter = (float)(magFieldBoundsRectangle.Width / 2.0);
			float yCenter = (float)(magFieldBoundsRectangle.Height / 2.0);

			float xOld = xCenter + (float)(mag.MagneticField[0]) * magFieldCircleRadius * (float)(1 / ambientMagneticField);
			float yOld = yCenter + (float)(mag.MagneticField[1]) * magFieldCircleRadius * (float)(1 / ambientMagneticField);

			magFieldGraphicsBuffer.Graphics.DrawLine(magFieldXYAxisPen, xCenter, yCenter, xOld, yOld);
			magFieldGraphicsBuffer.Graphics.DrawEllipse(magFieldCirclePen, magFieldCircleRectangle);

			double zOut = (mag.MagneticField[2]) * (float)(1 / ambientMagneticField);
			if (zOut > 0) {
				magFieldGraphicsBuffer.Graphics.DrawEllipse(new Pen(Color.Red, 2),
					new Rectangle((int)xCenter - (int)(magFieldCircleRadius * zOut), (int)yCenter - (int)(magFieldCircleRadius * zOut),
					(int)(magFieldCircleDiameter * zOut), (int)(magFieldCircleDiameter * zOut)));
			} else {
				magFieldGraphicsBuffer.Graphics.DrawEllipse(new Pen(Color.Green, 2),
					new Rectangle((int)xCenter - (int)(magFieldCircleRadius * -zOut), (int)yCenter - (int)(magFieldCircleRadius * -zOut),
					(int)(magFieldCircleDiameter * -zOut), (int)(magFieldCircleDiameter * -zOut)));
			}

			magFieldGraphicsBuffer.Render(magFieldView.CreateGraphics());

			magFieldGraphicsBuffer.Dispose();

			magFieldGraphicsContext.Dispose();
		}
		#endregion


	}
}
